import 'dart:core';
import 'package:flutter/cupertino.dart';
import 'package:i18n_plus/i18n.dart';
import 'package:i18n_plus/i18nLanguageset.dart';

///[i18nKey] must be a valid key in your current i18nMap
///[additionalString] can be added to your i18n value. If not null
///[otherLocale] fill field if you do not want to use the current locale
///All other params are the same as in the original [Text] widget
class I18NText extends StatefulWidget {
  final String? i18nKey;
  final String? additionalString;
  final Locale? otherLocale;
  final TextStyle? style;
  final StrutStyle? strutStyle;
  final TextAlign? textAlign;
  final TextDirection? textDirection;
  final bool? softWrap;
  final TextOverflow? overflow;
  final double? textScaleFactor;
  final int? maxLines;
  final String? semanticsLabel;
  final TextWidthBasis? textWidthBasis;
  final TextHeightBehavior? textHeightBehavior;

  const I18NText(
      {Key? key,
      this.i18nKey,
      this.style,
      this.strutStyle,
      this.textAlign,
      this.textDirection,
      this.softWrap,
      this.overflow,
      this.textScaleFactor,
      this.maxLines,
      this.semanticsLabel,
      this.textWidthBasis,
      this.textHeightBehavior,
      this.otherLocale,
      this.additionalString})
      : super(key: key);
  @override
  I18NTextState createState() => I18NTextState();
}

class I18NTextState extends State<I18NText> {
  final I18N _i18n = I18N();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder(
        valueListenable: _i18n.currentLocale,
        builder: (BuildContext context, Locale? value, Widget? child) {
          return Text(
              _textString(
                  widget.i18nKey, widget.additionalString, widget.otherLocale)!,
              textAlign: widget.textAlign,
              style: widget.style,
              strutStyle: widget.strutStyle,
              textDirection: widget.textDirection,
              softWrap: widget.softWrap,
              overflow: widget.overflow,
              textScaleFactor: widget.textScaleFactor,
              maxLines: widget.maxLines,
              semanticsLabel: widget.semanticsLabel,
              textWidthBasis: widget.textWidthBasis,
              textHeightBehavior: widget.textHeightBehavior);
        },
        child: null);
  }

  String? _textString(String? key, String? add, Locale? otherLocale) {
    add = add != null ? add : "";
    Langaugeset? wantedSet = _i18n.i18nMap!.getWithLocale(
        otherLocale != null && _i18n.allLocale()!.contains(widget.otherLocale)
            ? otherLocale
            : _i18n.currentLocale.value);
    if (wantedSet != null) {
      if (wantedSet.map[key] != null) {
        return wantedSet.map[key] + add;
      } else {
        return "Invalid Key";
      }
    }
  }
}
